//*****************************************************************************
//
// startup.c 
// 
// Setup the system clock and the ports.
//
//*****************************************************************************
#include <stdint.h>
#include <string.h>

#include <inc/hw_types.h>
#include <inc/hw_memmap.h>
#include <inc/hw_ssi.h>
#include <inc/hw_gpio.h>
#include <inc/hw_timer.h>
#include <inc/hw_ints.h>

#include <driverlib/gpio.h>
#include <driverlib/interrupt.h>
#include <driverlib/systick.h>
#include <driverlib/sysctl.h>
#include <driverlib/uart.h>
#include <driverlib/timer.h>

#define UART_BASE UART0_BASE
#define UART_PERIPH SYSCTL_PERIPH_UART0

#define UART_PORT_PERIPH SYSCTL_PERIPH_GPIOA
#define UART_PORT GPIO_PORTA_BASE
#define UART_BITS GPIO_PIN_0|GPIO_PIN_1

#define LED1_PERIPH SYSCTL_PERIPH_GPIOF
#define LED1_PORT GPIO_PORTF_BASE
#define LED1_BIT GPIO_PIN_0

#define CPUHZ 50000000
#define TICKHZ 1000

/*
 * Configure the processor 
 */
static void prvSetupHardware( void ) {
  	/* Setup the System Clock. 200 / 8 = 25MHz */ 
	SysCtlClockSet(SYSCTL_SYSDIV_4 |\
		SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_8MHZ);

	// Turn on Clock Gating control
	SysCtlPeripheralClockGating(true);

	// Enable the required GPIO Ports
	SysCtlPeripheralEnable(UART_PORT_PERIPH);  // GPIO Port for UART
	SysCtlPeripheralSleepEnable(UART_PORT_PERIPH); 
	GPIOPinTypeUART(UART_PORT,UART_BITS); 

	SysCtlPeripheralEnable(LED1_PERIPH); 
	SysCtlPeripheralSleepEnable(LED1_PERIPH); 
	GPIOPinTypeGPIOOutput(LED1_PORT,LED1_BIT);
 	GPIOPinWrite(LED1_PORT, LED1_BIT, LED1_BIT);

	SysCtlPeripheralEnable(UART_PERIPH);  
	SysCtlPeripheralSleepEnable(UART_PERIPH); 

	SysCtlPeripheralEnable(LED1_PERIPH); 

	UARTConfigSetExpClk(UART_BASE, CPUHZ, 9600,
		(UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |\
		 UART_CONFIG_PAR_NONE));

	UARTEnable(UART_BASE);

	// Setup the Stellaris Timer to serve as 
	// an activity load measurement device.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0); 
	SysCtlPeripheralSleepDisable(SYSCTL_PERIPH_TIMER0); 
	TimerConfigure(TIMER0_BASE, TIMER_CFG_32_BIT_PER);
	TimerLoadSet(TIMER0_BASE, TIMER_A, 0xffffffff);
	TimerEnable(TIMER0_BASE, TIMER_A);

	// These two are part of core debug. 
	// NOTE!  The Cycle counter runs during sleep in 
	// early revs of the Cortex-M3.  Check the ARM Errata.
	HWREG(0xE000EDFC) = 1<<24; // Set the TRCENA Bit to turn on the DWT
	HWREG(0xE0001000) = 1; 	   // Whack CYCCNTENA 

	}

//*****************************************************************************
// The Systick auto-rearms.  Just increment the counter.
//*****************************************************************************
unsigned int time_ms = 0;
unsigned int beforecount;
unsigned int aftercount;
unsigned int usage;

// Heres a knob for use by the delay routine.
// The delay code can can zero this out as needed.
unsigned int delayticker_ms;

void SysTickHandler() {
	time_ms++;
	if (time_ms == 1000) {
		time_ms = 0;
		// aftercount = HWREG(0xE0001004); // Arm Cycle Counter.
		// usage = aftercount - beforecount;

		// Load up the current value from the timer.
		aftercount = HWREG(TIMER0_BASE + TIMER_O_TAR); 
		usage = beforecount - aftercount;

		beforecount = aftercount;
		}
	delayticker_ms++;
	}

//*****************************************************************************
// Main - Set things up, launch forth
//*****************************************************************************
// int main (void) __attribute__((noreturn));

// int cworld = 0;	// Something that the linker can stash away and we can find.

int main(void) {

	// Setup the hardware.
	prvSetupHardware();

	char message[] = "Boot!\r\n";
	char* p;

	//*****************************************************************
	// Turn on SysTick
	SysTickPeriodSet(CPUHZ/TICKHZ); 
	SysTickEnable();
	// IntPrioritySet(FAULT_SYSTICK,(1<<5));
	SysTickIntEnable();

	// Turn on all interrupts
	IntMasterEnable();

	// cworld++;

	p = message;
	while(*p != 0) { 
		UARTCharPut(UART_BASE,*p);
		p++;
		}
	
	// Forth owns the user mode, and the prompt.  
	// The ISRs can hijack things, but thats perfectly ok, as they will
	// save system state.

	asm("b initForth");
		
	}

