        ;; skip the emacs mode line
;;; -*- Mode:Asm Mode:outline-minor-mode outline-regexp:";;;+" comment-start: "; " -*-
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; equates-lm3s.asm
        
        ;; The purpose of this file (equates-lm3s.asm) is to contain
        ;; all the code for Riscy Pygness that is specific to the
        ;; Luminary/TI Stellaris ARM M3 MCU.

        ;; This file (equates-lm3s.asm) is included by the
	;; riscy-lm3s.asm
	;; olimex-stm32-p103.asm).

;;; NOTE


        ;; Four subroutines and 2 Forth primitives must be defined.

        ;; The following subroutines must be defined, even if they do nothing,
        ;; because riscy.asm will call them.
        ;; 
        ;;      ledOnSub
        ;;      ledOffSub

        ;; Additional, application-specific CODE words can be defined in
        ;; this file.

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Different compilation environments have different reset vectors
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
platformRESET:
	b ResetISR

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Equates that do calculations and so are not likely to change
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; Frequency is given in number of clocks per second
	;; Fury-Class devices have a PLL and will run at 50Mhz

        .equ  PCLK, 50000000

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; The Following are handled by c2forth.c
;;; setup_clocks
;;; setup_ports
;;; setup_uarts
;;; 
;;; Almost everything in the Stellaris parts is defined via the 
;;; Stellarisware libraries.   This is an exception for the UART port,
;;; Which we will manipuate from forth using assembly.
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;;; Things lifted from the Stellaris includes.
	.equ UART0_BASE, 0x4000C000 
	.equ UART1_BASE, 0x4000D000 
	.equ UART2_BASE, 0x4000E000 
	.equ CONSOLE_BASE, UART0_BASE

	.equ UART_O_DR , 0x00000000  ;; UART Data Register
	.equ UART_O_FR , 0x00000018  ;; UART Flag
	.equ UART_FR_RXFE, 0x10	     ;; RX Fifo Empty
	.equ BIT_RXFE, 4	     ;; RX Fifo Empty
	.equ UART_FR_TXFF, 0x20	     ;; TX Fifo Full
	.equ BIT_TXFF, 5	     ;; TX Fifo Full

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; LED Subroutines - These values are adjusted for the LM3S6965 Eval board
;;; It has a user led on PF0
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.equ GPIO_PORTA_BASE, 0x40004000
	.equ GPIO_PORTB_BASE, 0x40005000
	.equ GPIO_PORTC_BASE, 0x40006000
	.equ GPIO_PORTD_BASE, 0x40007000
	.equ GPIO_PORTE_BASE, 0x40024000
	.equ GPIO_PORTF_BASE, 0x40025000
	.equ GPIO_PORTG_BASE, 0x40026000
	
	.equ LED_PORT, GPIO_PORTF_BASE
	.equ LED_MASK, 0x01


ledOnSub:
        ldr r0, = (LED_PORT + LED_MASK<<2)
        mov r1, # LED_MASK
        str r1, [r0]      
	mov pc, lr

ledOffSub:
        ldr r0, = (LED_PORT + LED_MASK<<2)
        mov r1, #0        ; LED bit 
        str r1, [r0]       
	mov pc, lr

;;;; Serial port subroutines - Board Specific.

; Subroutine to send the character in r1 to the serial port
; Character is still in r1 upon return. 
; 
; The Stellaris UART has a fifo, so we spin while waiting on 
; something to arrive in the FIFO.  Likewise, we spin 
; until there is room in the transmit FIFO.

tx:
	;; Load up the bit banded register for the uart fifo status
        ldr r3, = CONSOLE_BASE 
tdre:   
        ldr r2, [r3 , # UART_O_FR ]
        tst r2, # UART_FR_TXFF ; See if the fifo is full
        bne tdre          ; If its full, spin
        str  r0, [r3, # UART_O_DR] ; Write to serial port
        mov pc,lr

; Subroutine to read one character from the serial port 
rx:
        ldr r3, = CONSOLE_BASE 
rdr:   
        ldrb r1, [r3, # UART_O_FR]
        tst r1, # UART_FR_RXFE  ; See if the fifo is empty.
        bne rdr             ; If its empty, spin.
        ldrb r0, [r3, # UART_O_DR ]; Read character from serial port into r1
        mov  pc, lr             ; no, don't echo it, just return
        
        .ltorg  ; force dumping of literal pool

;;;; Serial port Forth words

;;;; label parenSERINQ is Forth word (SERIN?  ( - f)
; Answer true if a character is waiting in the serial port FIFO
; sometimes the assembler barfs on the bitband macro.
; bitbandload r6, (UART0_BASE+UART_O_FR), BIT_RXFE = 0x42180310
parenSERINQ:
        dup
	bitbandload r0, (CONSOLE_BASE+UART_O_FR), BIT_RXFE
        ldrb TOS, [r0] ;; 0 = Not Empty, 1 = Empty
	subs TOS, #1
        ;; We might use bit-banding instead.  This would would leave
        ;; the flag in bit0, then we could do 'sbc TOS, # 0' to
        ;; produce a canonical flag.  Or, we could shift TOS 4 bits to
        ;; the right, etc.
        nxt



;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Additional application-specific Forth Primitives could go here.
;;;
