/******************************************************************************
 *
 * hello.ld - Linker configuration file for hello.
 *
 * Copyright (c) 2006-2008 Luminary Micro, Inc.  All rights reserved.
 * 
 * Software License Agreement
 * 
 * Luminary Micro, Inc. (LMI) is supplying this software for use solely and
 * exclusively on LMI's microcontroller products.
 * 
 * The software is owned by LMI and/or its suppliers, and is protected under
 * applicable copyright laws.  All rights are reserved.  You may not combine
 * this software with "viral" open-source software in order to form a larger
 * program.  Any use in violation of the foregoing restrictions may subject
 * the user to criminal sanctions under applicable laws, as well as to civil
 * liability for the breach of the terms and conditions of this license.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 * 
 * This is part of revision 3223 of the Stellaris Peripheral Driver Library.
 *
 *****************************************************************************/

/* This linker map is hacked:
* 1: Skip the first 2k to make room for the boot loader.
* 2: Skip the last 2k to make room for flash pram.
*
* The LM3S608 has 32k of flash, and 8k of ram.
*
* There doesn't appear to be a good way to Automagically determine the
* stack boundaries so I just declare some lengths.  The M3 does
* a descending stack, so SP gets updated before the write.
* For doubleword support, AAPCS requires that the stack be 8-byte aligned.
*/

/* TUNABLES */

__primary_stack_size = 256;
__overflow_stack_size = 256;
/* You must adjust the sizes in the MEMORY declaration to match */
/* The linker doesn't like math declarations in the MEMORY * declaration. */

MEMORY
{
    FLASH (rx) : ORIGIN = 0k, LENGTH = 32k
    SRAM (rwx) : ORIGIN = 0x20000000, LENGTH = 8k - 512
    STACK (rx) : ORIGIN = 0x20000000 + 7k + 512, LENGTH = 512
}

/*
* The convention used by the Luminary micro linker stuff
* is that things get stuffed into sections, and the last
* thing that happens the the definition of _e<something>
* to mark the end
*/

SECTIONS
{
    .text :
    {
        _text = .;
        KEEP(*(.isr_vector))
        *(.text*)
        *(.rodata*)
        _etext = .;
    } > FLASH
    .data : AT(ADDR(.text) + SIZEOF(.text))
    /*  
     * We tag the beginning of RAM so that we can force things
     * to be put there by the linker, like fault handler data.
     *
     * The NVIC requires that an in-ram vector table be aligned
     * on a 1k boundary.
     * 
     * create a section after the C stuff where the linker can 
     * put the forth bits, so that the dictionaries can grow
     * upwards towards the machine stack.
     */
    {
        _data = .;
	*(SRAM_START) /* Tag the beginning for special stuff */
        *(.data*)
    	. = ALIGN(1024); /* In-RAM NVIC Vector table must be aligned */
        *(vtable)
        _edata = .;
    } > SRAM
    .bss :
    {
        _bss = .;
        *(.bss*)
        *(COMMON)
        _ebss = .;
    } > SRAM
  
   /* Riscy Pygness assumes that forth stuff lives in a section called .ram */
    .ram : 
    {
    	* (.ram)
    } > SRAM 

    /*
     * Stack is divided into two parts so we can use the MPU 
     * to detect stack overrun - Slotted into a fixed location
     */
    .stack : { 
    	. = ALIGN(8);
	_overflow_stack_start = .;
	. += __overflow_stack_size;

	. = ALIGN(8);
	_pri_stack_start = .;
	. += __primary_stack_size;
	_estack = .; 
	} >STACK
	
}
