#!/usr/bin/tclsh
package require Expect
set timeout 5

spawn ./r

# send "COLD\n"

# Put things in hex mode so we can check out bit ops.
expect ">"
send "HEX\n"

# Start with a basic stack checkout
expect ">"
send "1 2 3 4 5 6 7 8 9 10\n"
expect ">"
send ". . . . . . . . . .\n"
expect "10 9 8 7 6 5 4 3 2 1"

# Lets check out some stack operations
puts "Checking Stack Operators"

expect ">"
send "1 2 3 DUP . . . .\n"
expect "3 3 2 1"

expect ">"
send "1 2 3 SWAP . . . \n"
expect "2 3 1"

expect ">"
send "1 2 3 DROP . .\n"
expect "2 1"

expect ">"
send "1 2 3 2DROP .\n"
expect "1"

expect ">"
send "1 2 3 4 3DROP .\n"
expect "1"

expect ">"
send "1 2 3 NIP . .\n"
expect "3 1"

expect ">"
send "1 2 3 ROT . . .\n"
expect "1 3 2"
#;;;; label ROT is Forth word ROT  ( a b c - b c a)

expect ">"
send "5 6 OVER . . .\n"
expect "5 6 5"
# ;;;; label OVER is Forth word OVER ( a b - a b a)

expect ">"
send "1 2 3 4 2OVER . . . . . . \n"
expect "2 1 4 3 2 1 "
#;;;; label twoOVER is Forth word 2OVER ( a b c d - a b c d a b)

expect ">"
send "1 2 3 -ROT . . . \n"
expect "2 1 3"
#;;;; label minusROT is Forth word -ROT ( a b c - c a b)

expect ">"
send "1 2 3 4 2SWAP . . . .\n"
expect "2 1 4 3"
#;;;; label twoSWAP is Forth word 2SWAP ( a b c d - c d a b)

expect ">"
send "1 2 2DUP . . . .\n"
expect "2 1 2 1"
#;;;; label twoDUP is Forth word 2DUP  ( a b - a b a b)

expect ">"
send "1 2 3 3DUP . . .  . . . \n"
expect "3 2 1 3 2 1"
#;;;; label threeDUP is Forth word 3DUP  ( a b c - a b c a b c)

expect ">"
send "1 2 3 4 4DUP . . . .  . . . .\n"
expect "4 3 2 1 4 3 2 1"
#;;;; label fourDUP is Forth word 4DUP  ( a b c d - a b c d a b c d)

# Move data between the two stacks
puts "Moving data between stacks"

expect ">"
send "1 2 3 4 PUSH PUSH PUSH . POP POP POP . . .\n"
expect "1 4 3 2 ok"
#;;;; label PUSH is Forth word PUSH ( n -)
#;;;; label POP is Forth word POP ( - n)

expect ">"
send "1 2 3 4 5 2PUSH 2PUSH . 2POP 2POP  . . . .\n"
expect "1 5 4 3 2 ok"
#;;;; label twoPUSH is Forth word 2PUSH ( n n -)
#;;;; label twoPOP is Forth word 2POP ( - n n)

send "DECIMAL \n"
expect "ok"

puts "Basic Math operators"
expect ">"
send "5 1+  . 5 2+ . 5 1- . 5 2- . \n"
expect "6 7 4 3 ok"
#;;;; label onePlus is Forth word 1+ ( n - n+1)
#;;;; label twoPlus is Forth word 2+ ( n - n+2)
#;;;; label oneMinus is Forth word 1- ( n - n-1)
#;;;; label twoMinus is Forth word 2-  ( n - n-2)

send "HEX \n"
expect "ok"
expect ">"
send "b0e0 0e0f AND   b0e0 0e0f OR . . \n"
expect "BEEF 0"
#;;;; label AND is Forth word AND ( a b - u)
#;;;; label OR is Forth word OR ( a b - u)

expect ">"
send "12345678 SPLIT . . \n"
expect "1234 5678 ok"
#;;; label SPLIT is Forth word SPLIT ( aaaabbbb - 0000bbbb 0000aaaa)

expect ">"
send "12345678 ROR24 .\n"
expect "34567812"
#;;;; label ROR24 is Forth word ROR24  ( abcd - bcda)  rotate right 24 bits, putting msbyte into lsbyte

puts "Conditionals"
expect ">"
send "DECIMAL\n"
expect "ok"

expect ">"
send "1 2 > . 2 1 > . 1 1 < .   1 2 < . 2 1 < . 1 1 < . \n"
expect "0 -1 0 -1 0 0 ok"
#;;;; label lessThan is Forth word <  ( a b - f)
#;;;; label zeroGreaterThan is Forth word 0>  ( n - f)        
#;;;; label zeroLessThan is Forth word 0<  ( n - f)        
#;;;; label UgreaterThan is Forth word U>  ( a b - f)

expect ">"
send { -1 1 > . 1 -1 > . -1 -1 > .   -1 1 < . 1 -1 < . -1 -1 < . }
send "\n"
expect {0 -1 0 -1 0 0 ok}

expect ">"
send { -1 2 U> . -2 1 U> . -1 1 U< .   -1 2 U< . -2 1 U< . -1 1 U< . }
send "\n"
expect { -1 -1 0 0 0 0 ok}


puts "Stack Pointer Operators - Inspect Manually"
expect ">"
send "HEX\n"
expect "ok"

expect ">"
send "SP@ . 1 2 SP@ . . . \n"

expect ">"
send "RP@ . 1 2 2PUSH RP@ . 2POP . .  \n"

expect ">"
send "1 2 3 SP@ @ . . . . \n"
expect "2 3 2 1"

expect ">"
send "1 2 3 4 SP@ 5 SWAP ! . . . .  \n"
expect "4 5 2 1 ok"

#;;;; label SPfetch is Forth word SP@  ( - dataStkPtr)
#;;;; label RPfetch is Forth word RP@  ( - returnStkPtr)

expect ">"
send "SP@ . 1 2 3 4 SP@ . SP! SP@ .  \n"
expect "ok"

#;;; label SPstore is Forth word SP! ( -)  reload the data stack pointer



puts "Compilation Tests"

send ": compiletest 1 2 3 + + . ; \n"
expect "ok"
expect ">"
send "compiletest \n"
expect "6"
# ;;; label plus is Forth word +  ( a b - a+b)

expect ">"
send ": iftest  > IF .\" yes \" ELSE .\" no \" THEN ;\n"
expect "ok"
expect ">"
send "2 1 iftest 1 2 iftest\n"
expect "yes no"
expect "ok"
# ;;; label quote is Forth word quote ( - a)   in-line string

puts "Loop Primitives"

expect ">"
send ": looptest BEGIN 1 - DUP . DUP 0 = UNTIL ;\n"
expect "ok"
send "10 looptest\n"
expect "9 8 7 6 5 4 3 2 1 0 ok"

expect ">"
send ": looptest2 BEGIN 1 - DUP 0 > WHILE DUP . REPEAT ;\n"
expect "ok"
send "10 looptest2\n"
expect "9 8 7 6 5 4 3 2 1 ok"

expect ">"
send ": looptest3 FOR I . NEXT ;\n"
expect "ok"
send "10 looptest3\n"
expect "9 8 7 6 5 4 3 2 1 0 ok"


#;;;; label XOR is Forth word XOR ( a b - u)
#;;;; label equal is Forth word =  ( a b - f)
#;;;; label NOT is Forth word NOT
#;;;; label zeroEqual is Forth word 0=
#;;;; label greaterThan is Forth word >  ( a b - f)
# More advanced tests.

puts "\nComplete\n"

exit

# Unfinished business.
;;; label qDUP is Forth word ?DUP  ( 0 - 0)  or  ( nonZero - nonZero nonZero)
;;; label twoFetch is Forth word 2@  ( a - low32bits high32bits)
;;; label twoStore is Forth word 2!   ( low32 high32 a -)
;;; label parenSEROUT is Forth word (SEROUT  ( c -)
;;; label parenSERIN is Forth word (SERIN  ( - c)
;;; label Mplus is Forth word M+  ( a b - a+b c)
;;; label Dplus is Forth word D+ ( a b c d - a+c b+d+carry) PUSH SWAP PUSH M+ POP + POP + ; )
;;; label minus is Forth word -  ( a b - a-b)
;;; label twoTimes is Forth word 2*  ( a - 2a)
;;; label twoSlash is Forth word 2/  (a - a/2)  shift right one place, signed
;;; label U2slash is Forth word U2/  ( a - a/2) shift right one place, unsigned
;;; label twofiftysixslash is Forth word 256/  ( u - u/256)
;;; label plusStore is Forth word +! ( n a -)
;;; label plusUNDER is Forth word +UNDER  ( a b n - a+n b)
;;; label NEGATE is Forth word NEGATE ( n - -n)
;;; label DNEGATE is Forth word DNEGATE  ( d - -d) 
;;; label sixtyFourKTimes is Forth word 64K*   ( aaaabbbb - bbbb0000)  -- shift left 16 bits
;;; label sixtyFourKSlash is Forth word 64K/  ( aaaabbbb - 0000aaaa)  -- unsigned shift right 16 bits
;;; label UMtimes is Forth word UM*  ( u1 u2 - ud)
;;; label Mtimes is Forth word M*  ( n1 n2 - d)
;;; label UslashMOD is Forth word U/MOD ( dividend divisor - uR uQ) 
;;; label UlessThan is Forth word U<  ( a b - f)   ** like <  but we use carry rather than msbit for flag

;;; label abortquote is Forth word abortquote ( f -)  in-line string
;;; label LINK is Forth word LINK ( - a)  
;;; label PAUSE is Forth word PAUSE ( -)
;;; label COLD is Forth word COLD ( -)
;;; label RESET is Forth word RESET ( -)
;;; label dp is Forth word H  ( - a)
;;; label tickAbort is Forth word 'ABORT  ( - a)
;;; label queryinterpret is Forth word queryinterpret ( -)
;;; label qOK is Forth word ?ok  ( -)

;;; label COUNT is Forth word COUNT ( a - a+1 c)
;;; label TYPE is Forth word TYPE   ( a c - )
;;; label EMIT is Forth word EMIT   ( c - )
;;; label number is Forth word number ( n -)
;;; label EXECUTE is Forth word EXECUTE ( a -)
;;; label EXIT is Forth word EXIT (  -)
;;; label NOP is Forth word NOP (  -)
;;; label IAP is Forth word IAP ( a -)

# Multitasking related words - Testing won't tell us much.
;;;; label bg1 is Forth word TASK1 ( - a)
;;;; label bg2 is Forth word TASK2 ( - a)
;;;; label SP0 is Forth word SP0 ( - a)  
;;;; label RP0 is Forth word RP0 ( - a)  
;;;; label TERMINAL is Forth word TERMINAL ( - a)
;;;; label TERMINAL is Forth word FOREGROUND ( - a)

# Things that I won't write tests for just yet.
;;;; label Rfetch is Forth word R@  ( - a)
;;;; label twoRfetch is Forth word 2R@  ( - a b)
;;;; label times is Forth word *  ( a b - a*b)
;;; label RPstore is Forth word RP! ( -)  reload the return stack pointer

# Things that don't really need testing. 
;;;; label cpuClock is Forth word CCLK ( - u)        
;;;; label ticksPerMS is Forth word TICKS/MS ( - u)        

# No point testing char/halfword versions.
;;; label Wfetch is Forth word W@  ( a - unsigned16bits)   (little endian)
;;; label Wstore is Forth word W!   ( unsigned16bits a -)   (little endian)

# Items that are not amenable to testing.
;;; label for is Forth word for ( # -)
;;; label next is Forth word next
;;; label zeroBranch is Forth word 0branch        
;;; label branch is Forth word branch        
;;; label lit16 is Forth word lit16 ( - n)
;;; label lit32 is Forth word lit32 ( - n)

# Fomatting words.
;;; label dotR is Forth word .R   ( n # - )
;;; label Udot is Forth word U.   ( u - )
;;; label UdotR is Forth word U.R   ( u # - )

